import React, { useEffect, useRef, useState } from 'react'
import { Box, Grid } from '@mui/material'
import useMediaQuery from '@mui/material/useMediaQuery'
import { useRemoteUsers } from 'agora-rtc-react'
import { useTheme, type Theme } from '@mui/material/styles'
import type { IAgoraRTCRemoteUser, ICameraVideoTrack } from 'agora-rtc-sdk-ng'
import UserCard from '../UserCard/UserCard'
import { socketService } from '../../Services/SocketService/socketService'

interface Props {
  totalUsers: number
  gridContainerHeight: number
  isPinned: boolean
  setIsPinned: (value: boolean) => void
  setPinnedUser: (value: IAgoraRTCRemoteUser | null) => void
  localCameraTrack: ICameraVideoTrack | null
}

const SymmetricGrid = ({
  totalUsers,
  gridContainerHeight,
  isPinned,
  setIsPinned,
  setPinnedUser,
  localCameraTrack,
}: Props) => {
  const theme: Theme = useTheme()
  const isSm = useMediaQuery(theme.breakpoints.up('sm'))
  const isMd = useMediaQuery(theme.breakpoints.up('md'))
  const isLg = useMediaQuery(theme.breakpoints.up('lg'))

  const remoteUsers = useRemoteUsers()

  const videoRef = useRef<HTMLVideoElement>(null)
  const canvasRef = useRef<HTMLCanvasElement>(null)

  const [itemOffset, setItemOffset] = useState(12)
  const [itemHeight, setItemHeight] = useState(230)
  const [numOfItems, setNumOfItems] = useState(1)

  const maxUsersDisplay = isLg ? 12 : isMd ? 9 : isSm ? 4 : 2

  useEffect(() => {
    if (localCameraTrack && videoRef.current) {
      const stream = localCameraTrack.getMediaStreamTrack()
      videoRef.current.srcObject = new MediaStream([stream])
      videoRef.current.play().catch((err) => console.error('Video play failed', err))
    }
  }, [localCameraTrack])

  useEffect(() => {
    setNumOfItems(totalUsers > maxUsersDisplay ? maxUsersDisplay : totalUsers)
  }, [totalUsers])

  useEffect(() => {
    switch (numOfItems) {
      case 1:
        setItemOffset(12)
        setItemHeight(gridContainerHeight)
        break
      case 2:
        setItemOffset(isSm ? 6 : 12)
        setItemHeight(isSm ? gridContainerHeight : (gridContainerHeight - 16) / 2)
        break
      case 3:
        setItemOffset(isMd ? 4 : isSm ? 6 : 12)
        setItemHeight(
          isMd
            ? gridContainerHeight
            : isSm
              ? (gridContainerHeight - 16) / 2
              : (gridContainerHeight - 32) / 3,
        )
        break
      case 4:
        setItemOffset(isSm ? 6 : 12)
        setItemHeight(isSm ? (gridContainerHeight - 16) / 2 : (gridContainerHeight - 16 * 3) / 4)
        break
      case 5:
        setItemOffset(isMd ? 4 : isSm ? 6 : 12)
        setItemHeight(
          isMd
            ? (gridContainerHeight - 16) / 2
            : isSm
              ? (gridContainerHeight - 16 * 2) / 3
              : (gridContainerHeight - 32 * 4) / 5,
        )
        break
      case 6:
        setItemOffset(isMd ? 4 : isSm ? 6 : 12)
        setItemHeight(
          isMd
            ? (gridContainerHeight - 16) / 2
            : isSm
              ? (gridContainerHeight - 16 * 2) / 3
              : (gridContainerHeight - 32 * 5) / 6,
        )
        break
      case 7:
        setItemOffset(isLg ? 3 : isMd ? 4 : isSm ? 6 : 12)
        setItemHeight(
          isLg
            ? (gridContainerHeight - 16) / 2
            : isMd
              ? (gridContainerHeight - 16 * 2) / 3
              : isSm
                ? (gridContainerHeight - 16 * 3) / 4
                : (gridContainerHeight - 32 * 6) / 7,
        )
        break
      case 8:
        setItemOffset(isLg ? 3 : isMd ? 4 : isSm ? 6 : 12)
        setItemHeight(
          isLg
            ? (gridContainerHeight - 16) / 2
            : isMd
              ? (gridContainerHeight - 16 * 2) / 3
              : isSm
                ? (gridContainerHeight - 16 * 3) / 4
                : (gridContainerHeight - 32 * 7) / 8,
        )
        break
      case 9:
        setItemOffset(isMd ? 4 : isSm ? 6 : 12)
        setItemHeight(
          isMd
            ? (gridContainerHeight - 16 * 2) / 3
            : isSm
              ? (gridContainerHeight - 16 * 4) / 5
              : (gridContainerHeight - 32 * 8) / 9,
        )
        break
      default:
        setItemOffset(isLg ? 3 : isMd ? 4 : isSm ? 6 : 12)
        setItemHeight(
          isLg ? (gridContainerHeight - 16 * 2) / 3 : (gridContainerHeight - 16 * 3) / 4,
        )
    }
  }, [totalUsers, isSm, isMd, isLg, gridContainerHeight, numOfItems])

  const captureFrame = () => {
    if (videoRef.current && canvasRef.current) {
      const context = canvasRef.current.getContext('2d')
      if (context && videoRef.current.videoWidth && videoRef.current.videoHeight) {
        canvasRef.current.width = videoRef.current.videoWidth
        canvasRef.current.height = videoRef.current.videoHeight
        context.drawImage(videoRef.current, 0, 0, canvasRef.current.width, canvasRef.current.height)

        // You can now access the image data from the canvas
        const frame = canvasRef.current.toDataURL('image/jpeg')
        if (frame) {
          socketService.sendFrames('test', frame)
        }
      }
    }

    requestAnimationFrame(captureFrame) // Recursively capture the next frame
  }

  useEffect(() => {
    captureFrame()
  }, [])

  return (
    <Box sx={{ display: 'flex', width: '100%' }}>
      <Grid container spacing={3} width={'100%'} padding={0}>
        <Grid item xs={itemOffset}>
          <UserCard
            type={'local'}
            itemHeight={itemHeight}
            setIsPinned={setIsPinned}
            setPinnedUser={setPinnedUser}
            localCameraTrack={localCameraTrack}
          />
          <video ref={videoRef} style={{ display: 'none' }}></video>
          <canvas ref={canvasRef} style={{ display: 'none' }}></canvas>

          {/* <Box
            sx={{
              backgroundColor: '#262625',
              height: `${itemHeight}px`,
              borderRadius: '15px',
              width: '100%',
              position: 'relative',
              overflow: 'hidden',
            }}
          ></Box> */}
        </Grid>
        {remoteUsers.map((remoteUser, index) => (
          <Grid item xs={itemOffset} key={index}>
            <UserCard
              type={'remote'}
              remoteUser={remoteUser}
              itemHeight={itemHeight}
              setIsPinned={setIsPinned}
              setPinnedUser={setPinnedUser}
            />
          </Grid>
        ))}
      </Grid>
    </Box>
  )
}

export default SymmetricGrid
